#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <grp.h>
#include <string.h>
#include <fcntl.h>
#include "iolib.h"
#include "freedt.h"

void warn(const char *msg) {
	iformat(ierr, 'c', progname, 'c', ": ", 'c', msg, 'c', "\n", 0);
}

void warn2(const char *msg1, const char *msg2) {
	iformat(ierr, 'c', progname, 'c', ": ", 'c', msg1, 
		'c', ": ", 'c', msg2, 'c', "\n", 0);
}

void die(const char *msg) {
	warn(msg);
	exit(111);
}

void die2(const char *msg1, const char *msg2) {
	warn2(msg1, msg2);
	exit(111);
}

void setuidgidroot() {
	char *root, *uid, *gid;

	root = getenv("ROOT");
	if (root != NULL) {
		if (chdir(root) < 0)
			die2(root, "unable to chdir");
		if (chroot(root) < 0)
			die2(root, "unable to chroot");
	}

	gid = getenv("GID");
	if (gid != NULL) {
		gid_t g = atoi(gid);

		if (setgid(g) < 0)
			die("unable to setgid");
		if (setgroups(1, &g) < 0)
			die("unable to setgroups");
	}

	uid = getenv("UID");
	if (uid != NULL) {
		uid_t u = atoi(uid);

		if (setuid(u) < 0)
			die("unable to setuid");
	}
}

int fdt_setenv(const char *name, const char *value) {
	char *s = malloc(strlen(name) + strlen(value) + 2);
	if (s == NULL)
		return -1;
	strcpy(s, name);
	strcat(s, "=");
	strcat(s, value);
	if (putenv(s) < 0)
		return -1;
	return 0;
}

void change_fd_flags(int fd, int add, int remove) {
	int flags = fcntl(fd, F_GETFL);
	if (flags < 0)
		die("unable to read FD flags");

	flags |= add;
	flags &= ~remove;

	if (fcntl(fd, F_SETFL, flags) < 0)
		die("unable to set FD flags");
}

void set_fd_cloexec(int fd) {
	if (fcntl(fd, F_SETFD, FD_CLOEXEC) < 0)
		die("unable to set FD_CLOEXEC");
}

void reliable_sleep(int seconds) {
	while (seconds > 0)
		seconds = sleep(seconds);
}

