#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include "svstat.h"
#include "iolib.h"
const char *progname = "svstat";
#include "freedt.h"

#define FAIL(msg) { \
	iformat(iout, 'c', msg, 'c', "\n", 0); \
	continue; \
	}

int main(int argc, char **argv) {
	int fd, i, cwd;

	istdinit();

	cwd = open(".", O_RDONLY);
	if (cwd < 0) die("unable to open current directory");
		
	for (i = 1; i < argc; i++) {
		struct svstat s;
		int rc;
		long tup;

		iformat(iout, 'c', argv[i], 'c', ": ", 0);

		if (fchdir(cwd) < 0)
			FAIL("unable to chdir to original dir");
		if (chdir(argv[i]) < 0)
			FAIL("unable to chdir to service dir");

		fd = open("supervise/status", O_RDONLY);
		if (fd < 0) {
			FAIL("unable to open supervise/status");
		}
		rc = read(fd, &s, sizeof s);
		close(fd);
		if (rc < 0)
			FAIL("unable to read supervise/status");

		tup = time(NULL) - s.starttime;

		if (s.up) {
			iformat(iout, 'c', "up (pid ", 'i', s.pid,
					'c', ") ", 'l', tup,
					'c', " seconds\n", 0);
		} else {
			const char *updown = "up";
			fd = open("down", O_RDONLY);
			if (fd >= 0) {
				close(fd);
				updown = "down";
			}
			iformat(iout, 'c', "down ", 'l', tup,
				'c', " seconds, normally ",
				'c', updown, 'c', "\n", 0);
		}
	}

	close(cwd);

	return 0;
}

